@@ -1,7 +1,6 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-# Copyright 2012 Rackspace
+# Copyright (c)2012 Rackspace US, Inc.
 
 # All Rights Reserved.
 #
@@ -16,29 +15,40 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
+from __future__ import absolute_import, unicode_literals
 
 from functools import wraps
 import json
 import re
 import time
 
+import six
+
 import pyrax
 from pyrax.client import BaseClient
+from pyrax.cloudloadbalancers import CloudLoadBalancer
 import pyrax.exceptions as exc
 from pyrax.manager import BaseManager
 from pyrax.resource import BaseResource
 import pyrax.utils as utils
 
 # How long (in seconds) to wait for a response from async operations
-WAIT_LIMIT = 5
+DEFAULT_TIMEOUT = 5
+# How long (in seconds) to wait in between checks for async completion
+DEFAULT_DELAY = 0.5
+# How many times to retry a GET before raising an error
+DEFAULT_RETRY = 3
 
 
 def assure_domain(fnc):
     @wraps(fnc)
     def _wrapped(self, domain, *args, **kwargs):
         if not isinstance(domain, CloudDNSDomain):
-            # Must be the ID
-            domain = self._manager.get(domain)
+            # Must be the ID or name. Try ID first:
+            try:
+                domain = self._manager.get(domain)
+            except exc.NotFound:
+                domain = self._manager.find(name=domain)
         return fnc(self, domain, *args, **kwargs)
     return _wrapped
 
@@ -57,13 +67,12 @@ class CloudDNSRecord(BaseResource):
     ttl = None
     comment = None
 
-
     def update(self, data=None, priority=None, ttl=None, comment=None):
         """
         Modifies this record.
         """
         return self.manager.update_record(self.domain_id, self, data=data,
-                priority=priority, ttl=ttl, comment=comment)
+            priority=priority, ttl=ttl, comment=comment)
 
 
     def get(self):
@@ -130,7 +139,8 @@ class CloudDNSDomain(BaseResource):
         Sample export:
             {u'accountId': 000000,
              u'contentType': u'BIND_9',
-             u'contents': u'example.com.\t3600\tIN\tSOA\tns.rackspace.com. foo@example.com. 1354202974 21600 3600 1814400 500'
+             u'contents': u'example.com.\t3600\tIN\tSOA\tns.rackspace.com. '
+                'foo@example.com. 1354202974 21600 3600 1814400 500'
                 'example.com.\t3600\tIN\tNS\tdns1.stabletransit.com.'
                 'example.com.\t3600\tIN\tNS\tdns2.stabletransit.com.',
              u'id': 1111111}
@@ -221,7 +231,24 @@ class CloudDNSDomain(BaseResource):
         Modifies an existing record for this domain.
         """
         return self.manager.update_record(self, record, data=data,
-                priority=priority, ttl=ttl, comment=comment)
+            priority=priority, ttl=ttl, comment=comment)
+
+
+    def update_records(self, records):
+        """
+        Modifies multiple existing records for a domain. Each record to be
+        updated should be a dict with following required keys:
+            - id
+            - name
+
+        Each record must also contain one or more of the following optional
+        keys:
+            - data
+            - ttl
+            - comment
+            - priority (optional for MX and SRV records; forbidden otherwise)
+        """
+        return self.manager.update_records(self, records)
 
 
     def delete_record(self, record):
@@ -231,6 +258,7 @@ class CloudDNSDomain(BaseResource):
         return self.manager.delete_record(self, record)
 
 
+
 class CloudDNSPTRRecord(object):
     """
     This represents a Cloud DNS PTR record (reverse DNS).
@@ -266,6 +294,49 @@ class CloudDNSManager(BaseManager):
                 uri_base=uri_base)
         self._paging = {"domain": {}, "subdomain": {}, "record": {}}
         self._reset_paging(service="all")
+        self._timeout = DEFAULT_TIMEOUT
+        self._delay = DEFAULT_DELAY
+
+
+    def _create_body(self, name, emailAddress, ttl=3600, comment=None,
+            subdomains=None, records=None):
+        """
+        Creates the appropriate dict for creating a new domain.
+        """
+        if subdomains is None:
+            subdomains = []
+        if records is None:
+            records = []
+        body = {"domains": [{
+                "name": name,
+                "emailAddress": emailAddress,
+                "ttl": ttl,
+                "comment": comment,
+                "subdomains": {
+                    "domains": subdomains
+                    },
+                "recordsList": {
+                    "records": records
+                    },
+                }]}
+        return body
+
+
+    def _set_timeout(self, timeout):
+        """
+        Changes the duration for which the program will wait for a response from
+        the DNS system. Setting the timeout to zero will make that program wait
+        an indefinite amount of time.
+        """
+        self._timeout = timeout
+
+
+    def _set_delay(self, delay):
+        """
+        Changes the interval that the program will pause in between attempts to
+        see if a request has completed.
+        """
+        self._delay = delay
 
 
     def _reset_paging(self, service, body=None):
@@ -319,7 +390,7 @@ class CloudDNSManager(BaseManager):
         Handles the communication with the API when getting
         a full listing of the resources managed by this class.
         """
-        _resp, resp_body = self.api.method_get(uri)
+        resp, resp_body = self._retry_get(uri)
         if obj_class is None:
             obj_class = self.resource_class
 
@@ -371,10 +442,22 @@ class CloudDNSManager(BaseManager):
         the BaseManager method must be overridden here.
         """
         uri = "%s?showRecords=false&showSubdomains=false" % uri
-        _resp, body = self.api.method_get(uri)
+        resp, body = self._retry_get(uri)
         body["records"] = []
         return self.resource_class(self, body, loaded=True)
 
+    def _retry_get(self, uri):
+        """
+        Handles GET calls to the Cloud DNS API in order to retry on empty
+        body responses.
+        """
+        for i in six.moves.range(DEFAULT_RETRY):
+            resp, body = self.api.method_get(uri)
+            if body:
+                return resp, body
+        # Tried too many times
+        raise exc.ServiceResponseFailure("The Cloud DNS service failed to "
+                "respond to the request.")
 
     def _async_call(self, uri, body=None, method="GET", error_class=None,
             has_response=True, *args, **kwargs):
@@ -389,37 +472,53 @@ class CloudDNSManager(BaseManager):
         to handle the result.
         """
         api_methods = {
-                "GET": self.api.method_get,
+                "GET": self._retry_get,
                 "POST": self.api.method_post,
                 "PUT": self.api.method_put,
                 "DELETE": self.api.method_delete,
                 }
         api_method = api_methods[method]
-        if body is None:
-            _resp, ret_body = api_method(uri, *args, **kwargs)
-        else:
-            _resp, ret_body = api_method(uri, body=body, *args, **kwargs)
-        callbackURL = ret_body["callbackUrl"].split("/status/")[-1]
+        try:
+            if body is None:
+                resp, resp_body = api_method(uri, *args, **kwargs)
+            else:
+                resp, resp_body = api_method(uri, body=body, *args, **kwargs)
+        except Exception as e:
+            if error_class:
+                raise error_class(e)
+            else:
+                raise
+
+        callbackURL = resp_body["callbackUrl"].split("/status/")[-1]
         massagedURL = "/status/%s?showDetails=true" % callbackURL
         start = time.time()
-        while ((ret_body["status"] == "RUNNING") and
-                (time.time() - start < WAIT_LIMIT)):
-            _resp, ret_body = self.api.method_get(massagedURL)
-        if error_class and (ret_body["status"] == "ERROR"):
-            #This call will handle raising the error.
-            self._process_async_error(ret_body, error_class)
+        timed_out = False
+        while (resp_body["status"] == "RUNNING") and not timed_out:
+            resp_body = None
+            while resp_body is None and not timed_out:
+                resp, resp_body = self._retry_get(massagedURL)
+                if self._timeout:
+                    timed_out = ((time.time() - start) > self._timeout)
+                    time.sleep(self._delay)
+
+        if timed_out:
+            raise exc.DNSCallTimedOut("The API call to '%s' did not complete "
+                    "after %s seconds." % (uri, self._timeout))
+        if error_class and (resp_body["status"] == "ERROR"):
+            # This call will handle raising the error.
+            self._process_async_error(resp_body, error_class)
         if has_response:
-            ret = _resp, ret_body["response"]
+            ret = resp, resp_body["response"]
         else:
-            ret = _resp, ret_body
+            ret = resp, resp_body
         try:
-            body = json.loads(body)
+            resp_body = json.loads(resp_body)
         except Exception:
             pass
         return ret
 
 
-    def _process_async_error(self, ret_body, error_class):
+    def _process_async_error(self, resp_body, error_class):
         """
         The DNS API does not return a consistent format for their error
         messages. This abstracts out the differences in order to present
@@ -427,15 +526,15 @@ class CloudDNSManager(BaseManager):
         """
         def _fmt_error(err):
             # Remove the cumbersome Java-esque message
-            details = err["details"].replace("\n", " ")
+            details = err.get("details", "").replace("\n", " ")
             if not details:
-                details = err["message"]
-            return "%s (%s)" % (details, err["code"])
+                details = err.get("message", "")
+            return "%s (%s)" % (details, err.get("code", ""))
 
-        error = ret_body["error"]
+        error = resp_body.get("error", "")
         if "failedItems" in error:
             # Multi-error response
-            faults = error["failedItems"]["faults"]
+            faults = error.get("failedItems", {}).get("faults", [])
             msgs = [_fmt_error(fault) for fault in faults]
             msg = "\n".join(msgs)
         else:
@@ -468,9 +567,9 @@ class CloudDNSManager(BaseManager):
              "emailAddress": "sample@rackspace.com"}
         """
         self.run_hooks("modify_body_for_create", body, **kwargs)
-        _resp, ret_body = self._async_call(uri, body=body, method="POST",
+        resp, resp_body = self._async_call(uri, body=body, method="POST",
                 error_class=exc.DomainCreationFailed)
-        response_body = ret_body[self.response_key][0]
+        response_body = resp_body[self.response_key][0]
         return self.resource_class(self, response_body)
 
 
@@ -483,7 +582,7 @@ class CloudDNSManager(BaseManager):
         uri = "/%s/%s" % (self.uri_base, utils.get_id(domain))
         if delete_subdomains:
             uri = "%s?deleteSubdomains=true" % uri
-        _resp, ret_body = self._async_call(uri, method="DELETE",
+        resp, resp_body = self._async_call(uri, method="DELETE",
                 error_class=exc.DomainDeletionFailed, has_response=False)
 
 
@@ -498,11 +597,11 @@ class CloudDNSManager(BaseManager):
         """
         if (len(kwargs) == 1) and ("name" in kwargs):
             # Filtering on name; use the more efficient method.
-            nm = kwargs["name"]
+            nm = kwargs["name"].lower()
             uri = "/%s?name=%s" % (self.uri_base, nm)
             matches = self._list(uri, list_all=True)
             return [match for match in matches
-                if match.name == nm]
+                if match.name.lower() == nm]
         else:
             return super(CloudDNSManager, self).findall(**kwargs)
 
@@ -534,7 +633,7 @@ class CloudDNSManager(BaseManager):
         domain_id = utils.get_id(domain)
         dt = utils.iso_time_string(date_or_datetime, show_tzinfo=True)
         uri = "/domains/%s/changes?since=%s" % (domain_id, dt)
-        resp, body = self.api.method_get(uri)
+        resp, body = self._retry_get(uri)
         return body.get("changes", [])
 
 
@@ -547,15 +646,16 @@ class CloudDNSManager(BaseManager):
         Sample export:
             {u'accountId': 000000,
              u'contentType': u'BIND_9',
-             u'contents': u'example.com.\t3600\tIN\tSOA\tns.rackspace.com. foo@example.com. 1354202974 21600 3600 1814400 500'
+             u'contents': u'example.com.\t3600\tIN\tSOA\tns.rackspace.com. '
+                'foo@example.com. 1354202974 21600 3600 1814400 500'
                 'example.com.\t3600\tIN\tNS\tdns1.stabletransit.com.'
                 'example.com.\t3600\tIN\tNS\tdns2.stabletransit.com.',
              u'id': 1111111}
         """
         uri = "/domains/%s/export" % utils.get_id(domain)
-        resp, ret_body = self._async_call(uri, method="GET",
+        resp, resp_body = self._async_call(uri, method="GET",
                 error_class=exc.NotFound)
-        return ret_body.get("contents", "")
+        return resp_body.get("contents", "")
 
 
     def import_domain(self, domain_data):
@@ -568,9 +668,9 @@ class CloudDNSManager(BaseManager):
                 "contentType": "BIND_9",
                 "contents": domain_data,
                 }]}
-        resp, ret_body = self._async_call(uri, method="POST", body=body,
+        resp, resp_body = self._async_call(uri, method="POST", body=body,
                 error_class=exc.DomainCreationFailed)
-        return ret_body
+        return resp_body
 
 
     def update_domain(self, domain, emailAddress=None, ttl=None, comment=None):
@@ -593,9 +693,9 @@ class CloudDNSManager(BaseManager):
                 if val is None]
         for none_key in none_keys:
             body.pop(none_key)
-        resp, ret_body = self._async_call(uri, method="PUT", body=body,
+        resp, resp_body = self._async_call(uri, method="PUT", body=body,
                 error_class=exc.DomainUpdateFailed, has_response=False)
-        return ret_body
+        return resp_body
 
 
     def list_subdomains(self, domain, limit=None, offset=None):
@@ -613,7 +713,7 @@ class CloudDNSManager(BaseManager):
 
 
     def _list_subdomains(self, uri, domain_id):
-        resp, body = self.api.method_get(uri)
+        resp, body = self._retry_get(uri)
         self._reset_paging("subdomain", body)
         subdomains = body.get("domains", [])
         return [CloudDNSDomain(self, subdomain, loaded=False)
@@ -657,7 +757,7 @@ class CloudDNSManager(BaseManager):
 
 
     def _list_records(self, uri):
-        resp, body = self.api.method_get(uri)
+        resp, body = self._retry_get(uri)
         self._reset_paging("record", body)
         # The domain ID will be in the URL
         pat = "domains/([^/]+)/records"
@@ -710,12 +810,12 @@ class CloudDNSManager(BaseManager):
         uri = "/domains/%s/records?type=%s" % (dom_id, record_type)
         if query_string:
             uri = "%s&%s" % (uri, query_string)
-        resp, body = self.api.method_get(uri)
+        resp, body = self._retry_get(uri)
         records = body.get("records", [])
         self._reset_paging("record", body)
         rec_paging = self._paging.get("record", {})
         while rec_paging.get("next_uri"):
-            resp, body = self.api.method_get(rec_paging.get("next_uri"))
+            resp, body = self._retry_get(rec_paging.get("next_uri"))
             self._reset_paging("record", body)
             records.extend(body.get("records", []))
         for record in records:
@@ -741,9 +841,9 @@ class CloudDNSManager(BaseManager):
         dom_id = utils.get_id(domain)
         uri = "/domains/%s/records" % dom_id
         body = {"records": records}
-        resp, ret_body = self._async_call(uri, method="POST", body=body,
+        resp, resp_body = self._async_call(uri, method="POST", body=body,
                 error_class=exc.DomainRecordAdditionFailed, has_response=False)
-        records = ret_body.get("response", {}).get("records", [])
+        records = resp_body.get("response", {}).get("records", [])
         for record in records:
             record["domain_id"] = dom_id
         return [CloudDNSRecord(self, record, loaded=False)
@@ -755,9 +855,11 @@ class CloudDNSManager(BaseManager):
         Gets the full information for an existing record for this domain.
         """
         rec_id = utils.get_id(record)
-        uri = "/domains/%s/records/%s" % (utils.get_id(domain), rec_id)
-        resp, ret_body = self.api.method_get(uri)
-        return ret_body
+        domain_id = utils.get_id(domain)
+        uri = "/domains/%s/records/%s" % (domain_id, rec_id)
+        resp, resp_body = self._retry_get(uri)
+        resp_body["domain_id"] = domain_id
+        return CloudDNSRecord(self, resp_body, loaded=False)
 
 
     def update_record(self, domain, record, data=None, priority=None,
@@ -765,16 +867,29 @@ class CloudDNSManager(BaseManager):
         """
         Modifies an existing record for a domain.
         """
-        rec_id = utils.get_id(record)
-        uri = "/domains/%s/records/%s" % (utils.get_id(domain), rec_id)
-        body = {"name": record.name}
-        all_opts = (("data", data), ("priority", priority), ("ttl", ttl),
-                ("comment", comment))
-        opts = [(k, v) for k, v in all_opts if v is not None]
-        body.update(dict(opts))
-        resp, ret_body = self._async_call(uri, method="PUT", body=body,
+        rdict = {"id": record.id,
+                "name": record.name,
+                }
+        pdict = {"data": data,
+                "priority": priority,
+                "ttl": ttl,
+                "comment": comment,
+                }
+        utils.params_to_dict(pdict, rdict)
+        return self.update_records(domain, [rdict])
+
+
+    def update_records(self, domain, records):
+        """
+        Modifies an existing records for a domain.
+        """
+        if not isinstance(records, list):
+            raise TypeError("Expected records of type list")
+        uri = "/domains/%s/records" % utils.get_id(domain)
+        resp, resp_body = self._async_call(uri, method="PUT",
+                body={"records": records},
                 error_class=exc.DomainRecordUpdateFailed, has_response=False)
-        return ret_body
+        return resp_body
 
 
     def delete_record(self, domain, record):
@@ -783,9 +898,9 @@ class CloudDNSManager(BaseManager):
         """
         uri = "/domains/%s/records/%s" % (utils.get_id(domain),
                 utils.get_id(record))
-        resp, ret_body = self._async_call(uri, method="DELETE",
+        resp, resp_body = self._async_call(uri, method="DELETE",
                 error_class=exc.DomainRecordDeletionFailed, has_response=False)
-        return ret_body
+        return resp_body
 
 
     def _get_ptr_details(self, device, device_type):
@@ -793,12 +908,14 @@ class CloudDNSManager(BaseManager):
         Takes a device and device type and returns the corresponding HREF link
         and service name for use with PTR record management.
         """
+        context = self.api.identity
+        region = self.api.region_name
         if device_type.lower().startswith("load"):
-            ep = pyrax._get_service_endpoint("load_balancer")
+            ep = pyrax._get_service_endpoint(context, "load_balancer", region)
             svc = "loadbalancers"
             svc_name = "cloudLoadBalancers"
         else:
-            ep = pyrax._get_service_endpoint("compute")
+            ep = pyrax._get_service_endpoint(context, "compute", region)
             svc = "servers"
             svc_name = "cloudServersOpenStack"
         href = "%s/%s/%s" % (ep, svc, utils.get_id(device))
@@ -810,12 +927,18 @@ class CloudDNSManager(BaseManager):
         Given a device, determines if it is a CloudServer, a CloudLoadBalancer,
         or an invalid device.
         """
-        from tests.unit import fakes
-        if isinstance(device, (pyrax.CloudServer, fakes.FakeServer,
-                fakes.FakeDNSDevice)):
+        try:
+            from tests.unit import fakes
+            server_types = (pyrax.CloudServer, fakes.FakeServer)
+            lb_types = (CloudLoadBalancer, fakes.FakeLoadBalancer,
+                    fakes.FakeDNSDevice)
+        except ImportError:
+            # Not running with tests
+            server_types = (pyrax.CloudServer, )
+            lb_types = (CloudLoadBalancer, )
+        if isinstance(device, server_types):
             device_type = "server"
-        elif isinstance(device, (pyrax.CloudLoadBalancer,
-                fakes.FakeLoadBalancer)):
+        elif isinstance(device, lb_types):
             device_type = "loadbalancer"
         else:
             raise exc.InvalidDeviceType("The device '%s' must be a CloudServer "
@@ -831,11 +954,11 @@ class CloudDNSManager(BaseManager):
         href, svc_name = self._get_ptr_details(device, device_type)
         uri = "/rdns/%s?href=%s" % (svc_name, href)
         try:
-            resp, ret_body = self.api.method_get(uri)
+            resp, resp_body = self._retry_get(uri)
         except exc.NotFound:
             return []
         records = [CloudDNSPTRRecord(rec, device)
-                for rec in ret_body.get("records", [])]
+                for rec in resp_body.get("records", [])]
         return records
 
 
@@ -867,14 +990,14 @@ class CloudDNSManager(BaseManager):
         # The Rackspace DNS team is working on changing this to return a 403
         # instead; when that happens this kludge can go away.
         try:
-            _resp, ret_body = self._async_call(uri, body=body, method="POST",
+            resp, resp_body = self._async_call(uri, body=body, method="POST",
                     error_class=exc.PTRRecordCreationFailed)
         except exc.EndpointNotFound:
             raise exc.InvalidPTRRecord("The domain/IP address information is not "
                     "valid for this device.")
-        return ret_body.get("records")
+        return resp_body.get("records")
         records = [CloudDNSPTRRecord(rec, device)
-                for rec in ret_body.get("records", [])]
+                for rec in resp_body.get("records", [])]
         return records
 
 
@@ -909,12 +1032,12 @@ class CloudDNSManager(BaseManager):
                 }}
         uri = "/rdns"
         try:
-            _resp, ret_body = self._async_call(uri, body=body, method="PUT",
+            resp, resp_body = self._async_call(uri, body=body, method="PUT",
                     has_response=False, error_class=exc.PTRRecordUpdateFailed)
         except exc.EndpointNotFound as e:
             raise exc.InvalidPTRRecord("The record domain/IP address "
                     "information is not valid for this device.")
-        return ret_body.get("status") == "COMPLETED"
+        return resp_body.get("status") == "COMPLETED"
 
 
     def delete_ptr_records(self, device, ip_address=None):
@@ -927,17 +1050,19 @@ class CloudDNSManager(BaseManager):
         uri = "/rdns/%s?href=%s" % (svc_name, href)
         if ip_address:
             uri = "%s&ip=%s" % (uri, ip_address)
-        _resp, ret_body = self._async_call(uri, method="DELETE",
+        resp, resp_body = self._async_call(uri, method="DELETE",
                 has_response=False,
                 error_class=exc.PTRRecordDeletionFailed)
-        return ret_body.get("status") == "COMPLETED"
+        return resp_body.get("status") == "COMPLETED"
 
 
 
 class CloudDNSClient(BaseClient):
     """
-    This is the primary class for interacting with Cloud Databases.
+    This is the primary class for interacting with Cloud DNS.
     """
+    name = "Cloud DNS"
+
     def _configure_manager(self):
         """
         Creates a manager to handle the instances, and another
@@ -947,29 +1072,34 @@ class CloudDNSClient(BaseClient):
                 response_key="domains", plural_response_key="domains",
                 uri_base="domains")
 
+    def method_get(self, uri, **kwargs):
+        """
+        Overload the method_get function in order to retry on empty body
+        responses from the Cloud DNS API
+        """
+        for i in six.moves.range(3):
+            resp, body = super(CloudDNSClient, self).method_get(uri, **kwargs)
+            if body:
+                return resp, body
+        raise exc.ServiceResponseFailure("The Cloud DNS service failed to "
+                "respond to the request.")
 
-    def _create_body(self, name, emailAddress, ttl=3600, comment=None,
-            subdomains=None, records=None):
+
+    def set_timeout(self, timeout):
         """
-        Creates the appropriate dict for creating a new domain.
+        Sets the amount of time that calls will wait for a response from
+        the DNS system before timing out. Setting the timeout to zero will
+        cause execution to wait indefinitely until the call completes.
         """
-        if subdomains is None:
-            subdomains = []
-        if records is None:
-            records = []
-        body = {"domains": [{
-                "name": name,
-                "emailAddress": emailAddress,
-                "ttl": ttl,
-                "comment": comment,
-                "subdomains": {
-                    "domains": subdomains
-                    },
-                "recordsList": {
-                    "records": records
-                    },
-                }]}
-        return body
+        self._manager._set_timeout(timeout)
+
+
+    def set_delay(self, delay):
+        """
+        Changes the interval that the program will pause in between attempts to
+        see if a request has completed.
+        """
+        self._manager._set_delay(delay)
 
 
     def list(self, limit=None, offset=None):
@@ -1035,7 +1165,8 @@ class CloudDNSClient(BaseClient):
 
             {u'accountId': 000000,
              u'contentType': u'BIND_9',
-             u'contents': u'example.com.\t3600\tIN\tSOA\tns.rackspace.com. foo@example.com. 1354202974 21600 3600 1814400 500'
+             u'contents': u'example.com.\t3600\tIN\tSOA\tns.rackspace.com. '
+                'foo@example.com. 1354202974 21600 3600 1814400 500'
                 'example.com.\t3600\tIN\tNS\tdns1.stabletransit.com.'
                 'example.com.\t3600\tIN\tNS\tdns2.stabletransit.com.',
              u'id': 1111111}
@@ -1168,18 +1299,45 @@ class CloudDNSClient(BaseClient):
         """
         return domain.add_records(records)
 
-    #Create an alias, so that adding a single record is more intuitive
+    # Create an alias, so that adding a single record is more intuitive
     add_record = add_records
 
 
     @assure_domain
-    def update_record(self, domain, record, data=None, priority=None,
-            ttl=None, comment=None):
+    def get_record(self, domain, record):
+        """
+        Gets the full information for an existing record or record ID for the
+        specified domain.
+        """
+        return domain.get_record(record)
+
+
+    @assure_domain
+    def update_record(self, domain, record, data=None, priority=None, ttl=None,
+            comment=None):
         """
         Modifies an existing record for a domain.
         """
-        return domain.update_record(record, data=data,
-                priority=priority, ttl=ttl, comment=comment)
+        return domain.update_record(record, data=data, priority=priority,
+                ttl=ttl, comment=comment)
+
+
+    @assure_domain
+    def update_records(self, domain, records):
+        """
+        Modifies multiple existing records for a domain. Each record to be
+        updated should be a dict with following required keys:
+            - id
+            - name
+
+        Each record must also contain one or more of the following optional
+        keys:
+            - data
+            - ttl
+            - comment
+            - priority (optional for MX and SRV records; forbidden otherwise)
+        """
+        return domain.update_records(records)
 
 
     @assure_domain
