@@ -1,7 +1,6 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-# Copyright 2012 Rackspace
+# Copyright (c)2012 Rackspace US, Inc.
 
 # All Rights Reserved.
 #
@@ -16,10 +15,13 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
+from __future__ import absolute_import, unicode_literals
 
-import datetime
 from functools import wraps
 
+import six
+
+import pyrax
 from pyrax.client import BaseClient
 import pyrax.exceptions as exc
 from pyrax.manager import BaseManager
@@ -57,6 +59,7 @@ class CloudLoadBalancer(BaseResource):
         self._connection_logging = None
         self._content_caching = None
         self._session_persistence = None
+        self._non_display = ["nodes", "virtual_ips"]
         super(CloudLoadBalancer, self).__init__(*args, **kwargs)
 
 
@@ -70,9 +73,27 @@ class CloudLoadBalancer(BaseResource):
         return self.manager.add_virtualip(self, vip)
 
 
+    def get_usage(self, start=None, end=None):
+        """
+        Return the usage records for this load balancer. You may optionally
+        include a start datetime or an end datetime, or both, which will limit
+        the records to those on or after the start time, and those before or on
+        the end time. These times should be Python datetime.datetime objects,
+        Python datetime.date objects, or strings in the format:
+        "YYYY-MM-DD HH:MM:SS" or "YYYY-MM-DD".
+        """
+        return self.manager.get_usage(self, start=start, end=end)
+
+
+    def get_stats(self):
+        """
+        Return the stats for this loadbalancer
+        """
+        return self.manager.get_stats(self)
+
     def _add_details(self, info):
         """Override the base behavior to add Nodes, VirtualIPs, etc."""
-        for (key, val) in info.iteritems():
+        for (key, val) in six.iteritems(info):
             if key == "nodes":
                 val = [Node(parent=self, **nd) for nd in val]
             elif key == "sessionPersistence":
@@ -85,6 +106,23 @@ class CloudLoadBalancer(BaseResource):
             setattr(self, key, val)
 
 
+    def update(self, name=None, algorithm=None, protocol=None, halfClosed=None,
+            port=None, timeout=None, httpsRedirect=None):
+        """
+        Provides a way to modify the following attributes of a load balancer:
+            - name
+            - algorithm
+            - protocol
+            - halfClosed
+            - port
+            - timeout
+            - httpsRedirect
+        """
+        return self.manager.update(self, name=name, algorithm=algorithm,
+                protocol=protocol, halfClosed=halfClosed, port=port,
+                timeout=timeout, httpsRedirect=httpsRedirect)
+
+
     def delete_node(self, node):
         """Removes the node from the load balancer."""
         return self.manager.delete_node(self, node)
@@ -338,7 +376,7 @@ class CloudLoadBalancer(BaseResource):
         return self.manager.clear_error_page(self)
 
 
-    ## BEGIN - property definitions ##
+    # BEGIN - property definitions ##
     def _get_connection_logging(self):
         if self._connection_logging is None:
             self._connection_logging = self.manager.get_connection_logging(self)
@@ -366,7 +404,7 @@ class CloudLoadBalancer(BaseResource):
 
     def _set_session_persistence(self, val):
         if val:
-            if not isinstance(val, basestring) or (val.upper() not in
+            if not isinstance(val, six.string_types) or (val.upper() not in
                     ("HTTP_COOKIE", "SOURCE_IP")):
                 raise exc.InvalidSessionPersistenceType("Session Persistence "
                         "must be one of 'HTTP_COOKIE' or 'SOURCE_IP'. '%s' is "
@@ -388,11 +426,99 @@ class CloudLoadBalancer(BaseResource):
             _set_session_persistence, None, "The current state of session "
             "persistence. Possible values are either 'HTTP_COOKIE' or "
             "'SOURCE_IP', depending on the type of load balancing.")
-    ## END - property definitions ##
+    # END - property definitions ##
 
 
 
 class CloudLoadBalancerManager(BaseManager):
+    def update(self, lb, name=None, algorithm=None, protocol=None,
+            halfClosed=None, port=None, timeout=None, httpsRedirect=None):
+        """
+        Provides a way to modify the following attributes of a load balancer:
+            - name
+            - algorithm
+            - protocol
+            - halfClosed
+            - port
+            - timeout
+            - httpsRedirect
+        """
+        body = {}
+        if name is not None:
+            body["name"] = name
+        if algorithm is not None:
+            body["algorithm"] = algorithm
+        if protocol is not None:
+            body["protocol"] = protocol
+        if halfClosed is not None:
+            body["halfClosed"] = halfClosed
+        if port is not None:
+            body["port"] = port
+        if timeout is not None:
+            body["timeout"] = timeout
+        if httpsRedirect is not None:
+            body["httpsRedirect"] = httpsRedirect
+        if not body:
+            # Nothing passed
+            return
+        body = {"loadBalancer": body}
+        uri = "/loadbalancers/%s" % utils.get_id(lb)
+        try:
+            resp, resp_body = self.api.method_put(uri, body=body)
+        except exc.ClientException as e:
+            message = e.message
+            details = e.details
+            if message and details:
+                errmsg = "%s - %s" % (message, details)
+            else:
+                errmsg = message
+            raise exc.InvalidLoadBalancerParameters(errmsg)
+        return resp, resp_body
+
+
+    def _create_body(self, name, port=None, protocol=None, nodes=None,
+            virtual_ips=None, algorithm=None, halfClosed=None, accessList=None,
+            connectionLogging=None, connectionThrottle=None, healthMonitor=None,
+            metadata=None, timeout=None, sessionPersistence=None,
+            httpsRedirect=None):
+        """
+        Used to create the dict required to create a load balancer instance.
+        """
+        required = (virtual_ips, port, protocol)
+        if not all(required):
+            raise exc.MissingLoadBalancerParameters("Load Balancer creation "
+                "requires at least one virtual IP, a protocol, and a port.")
+        nodes = utils.coerce_to_list(nodes)
+        virtual_ips = utils.coerce_to_list(virtual_ips)
+        bad_conditions = [node.condition for node in nodes
+                if node.condition.upper() not in ("ENABLED", "DISABLED")]
+        if bad_conditions:
+            raise exc.InvalidNodeCondition("Nodes for new load balancer must be "
+                    "created in either 'ENABLED' or 'DISABLED' condition; "
+                    "received the following invalid conditions: %s" %
+                    ", ".join(set(bad_conditions)))
+        node_dicts = [nd.to_dict() for nd in nodes]
+        vip_dicts = [vip.to_dict() for vip in virtual_ips]
+        body = {"loadBalancer": {
+                "name": name,
+                "port": port,
+                "protocol": protocol,
+                "nodes": node_dicts,
+                "virtualIps": vip_dicts,
+                "algorithm": algorithm or "RANDOM",
+                "halfClosed": halfClosed,
+                "accessList": accessList,
+                "connectionLogging": connectionLogging,
+                "connectionThrottle": connectionThrottle,
+                "healthMonitor": healthMonitor,
+                "metadata": metadata,
+                "timeout": timeout,
+                "sessionPersistence": sessionPersistence,
+                "httpsRedirect": httpsRedirect,
+                }}
+        return body
+
+
     def add_nodes(self, lb, nodes):
         """Adds the list of nodes to the specified load balancer."""
         if not isinstance(nodes, (list, tuple)):
@@ -420,7 +546,6 @@ class CloudLoadBalancerManager(BaseManager):
         if not lb:
             raise exc.UnattachedNode("No parent Load Balancer for this node "
                     "could be determined.")
-
         if diff is None:
             diff = node._diff()
         req_body = {"node": diff}
@@ -837,7 +962,7 @@ class CloudLoadBalancerManager(BaseManager):
         return body
 
 
-    def get_stats(self, loadbalancer):
+    def get_stats(self, loadbalancer=None):
         """
         Returns statistics for the given load balancer.
         """
@@ -964,12 +1089,20 @@ class Node(object):
         return tmp % (self.type, self.condition, self.id, self.address, self.port,
                 self.weight)
 
+    def __eq__(self, other):
+        return (isinstance(other, self.__class__) and
+                self.__dict__ == other.__dict__)
+
+    def __ne__(self, other):
+        return not self.__eq__(other)
 
     def to_dict(self):
         """Convert this Node to a dict representation for passing to the API."""
         return {"address": self.address,
                 "port": self.port,
                 "condition": self.condition,
+                "type": self.type,
+                "id": self.id,
                 }
 
 
@@ -977,7 +1110,7 @@ class Node(object):
         """
         Returns the current metadata for the node.
         """
-        return self.manager.get_metadata(self, node=self)
+        return self.parent.get_metadata_for_node(self)
 
 
     def set_metadata(self, metadata):
@@ -985,7 +1118,7 @@ class Node(object):
         Sets the metadata for the node to the supplied dictionary
         of values. Any existing metadata is cleared.
         """
-        return self.manager.set_metadata(self, metadata, node=self)
+        return self.parent.set_metadata_for_node(self, metadata)
 
 
     def update_metadata(self, metadata):
@@ -993,7 +1126,7 @@ class Node(object):
         Updates the existing metadata for the node with
         the supplied dictionary.
         """
-        return self.manager.update_metadata(self, metadata, node=self)
+        return self.parent.update_metadata_for_node(self, metadata)
 
 
     def delete_metadata(self, keys=None):
@@ -1002,7 +1135,7 @@ class Node(object):
         this node. If no value for 'keys' is provided, all
         metadata is deleted.
         """
-        return self.manager.delete_metadata(self, keys=keys, node=self)
+        return self.parent.delete_metadata_for_node(self, keys=keys)
 
 
     @assure_parent
@@ -1028,11 +1161,25 @@ class Node(object):
         """
         diff = self._diff()
         if not diff:
-            #Nothing to do!
+            # Nothing to do!
             return
         self.parent.update_node(self, diff)
 
 
+    def get_device(self):
+        """
+        Returns a reference to the device that is represented by this node.
+        Returns None if no such device can be determined.
+        """
+        addr = self.address
+        servers = [server for server in pyrax.cloudservers.list()
+                if addr in server.networks.get("private", "")]
+        try:
+            return servers[0]
+        except IndexError:
+            return None
+
+
 
 class VirtualIP(object):
     """Represents a Virtual IP for a Load Balancer."""
@@ -1065,8 +1212,9 @@ class VirtualIP(object):
         Convert this VirtualIP to a dict representation for passing
         to the API.
         """
-        return {"type": self.type,
-                "ipVersion": self.ip_version}
+        if self.id:
+            return {"id": self.id}
+        return {"type": self.type, "ipVersion": self.ip_version}
 
 
     @assure_parent
@@ -1079,6 +1227,8 @@ class CloudLoadBalancerClient(BaseClient):
     """
     This is the primary class for interacting with Cloud Load Balancers.
     """
+    name = "Cloud Load Balancers"
+
     def __init__(self, *args, **kwargs):
         # Bring these two classes into the Client namespace
         self.Node = Node
@@ -1099,46 +1249,6 @@ class CloudLoadBalancerClient(BaseClient):
                 response_key="loadBalancer", uri_base="loadbalancers")
 
 
-    def _create_body(self, name, port=None, protocol=None, nodes=None,
-            virtual_ips=None, algorithm=None, halfClosed=None, accessList=None,
-            connectionLogging=None, connectionThrottle=None, healthMonitor=None,
-            metadata=None, timeout=None, sessionPersistence=None):
-        """
-        Used to create the dict required to create a load balancer instance.
-        """
-        required = (nodes, virtual_ips, port, protocol)
-        if not all(required):
-            raise exc.MissingLoadBalancerParameters("Load Balancer creation "
-                    "requires at least one node, one virtual IP, "
-                    "a protocol, and a port.")
-        bad_conditions = [node.condition for node in nodes
-                if node.condition.upper() not in ("ENABLED", "DISABLED")]
-        if bad_conditions:
-            raise exc.InvalidNodeCondition("Nodes for new load balancer must be "
-                    "created in either 'ENABLED' or 'DISABLED' condition; "
-                    "received the following invalid conditions: %s" %
-                    ", ".join(set(bad_conditions)))
-        node_dicts = [nd.to_dict() for nd in nodes]
-        vip_dicts = [vip.to_dict() for vip in virtual_ips]
-        body = {"loadBalancer": {
-                "name": name,
-                "port": port,
-                "protocol": protocol,
-                "nodes": node_dicts,
-                "virtualIps": vip_dicts,
-                "algorithm": algorithm or "RANDOM",
-                "halfClosed": halfClosed,
-                "accessList": accessList,
-                "connectionLogging": connectionLogging,
-                "connectionThrottle": connectionThrottle,
-                "healthMonitor": healthMonitor,
-                "metadata": metadata,
-                "timeout": timeout,
-                "sessionPersistence": sessionPersistence,
-                }}
-        return body
-
-
     def get_usage(self, loadbalancer=None, start=None, end=None):
         """
         Return the load balancer usage records for this account. If 'loadbalancer'
@@ -1198,6 +1308,24 @@ class CloudLoadBalancerClient(BaseClient):
         return self._protocols
 
 
+    @assure_loadbalancer
+    def update(self, loadbalancer, name=None, algorithm=None, protocol=None,
+            halfClosed=None, port=None, timeout=None, httpsRedirect=None):
+        """
+        Provides a way to modify the following attributes of a load balancer:
+            - name
+            - algorithm
+            - protocol
+            - halfClosed
+            - port
+            - timeout
+            - httpsRedirect
+        """
+        return self._manager.update(loadbalancer, name=name,
+                algorithm=algorithm, protocol=protocol, halfClosed=halfClosed,
+                port=port, timeout=timeout, httpsRedirect=httpsRedirect)
+
+
     @assure_loadbalancer
     def add_nodes(self, loadbalancer, nodes):
         """Adds the nodes to this load balancer."""
@@ -1531,4 +1659,4 @@ class CloudLoadBalancerClient(BaseClient):
         """
         loadbalancer.session_persistence = val
 
-    ## END pass-through methods ##
+    # END pass-through methods ##
