@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# Copyright 2012 Rackspace
+# Copyright (c)2012 Rackspace US, Inc.
 
 # All Rights Reserved.
 #
@@ -16,20 +16,23 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from __future__ import print_function
+
 import os
 import pyrax
 
+pyrax.set_setting("identity_type", "rackspace")
 creds_file = os.path.expanduser("~/.rackspace_cloud_credentials")
 pyrax.set_credential_file(creds_file)
 cbs = pyrax.cloud_blockstorage
-vol_name = pyrax.utils.random_name(length=8)
+vol_name = pyrax.utils.random_ascii(length=8)
 
 sata_vol = cbs.create(name="my_standard_volume", size=500, volume_type="SATA")
 ssd_vol = cbs.create(name="my_fast_volume", size=500, volume_type="SSD")
 
-print "SATA:", sata_vol
-print
-print "SSD:", ssd_vol
-print
-print "To delete these volumes, run 'delete_volume.py'"
-print
+print("SATA:", sata_vol)
+print()
+print("SSD:", ssd_vol)
+print()
+print("To delete these volumes, run 'delete_volume.py'")
+print()
