@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# Copyright 2012 Rackspace
+# Copyright (c)2012 Rackspace US, Inc.
 
 # All Rights Reserved.
 #
@@ -16,11 +16,14 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from __future__ import print_function
+
 import os
 import sys
 
 import pyrax
 
+pyrax.set_setting("identity_type", "rackspace")
 creds_file = os.path.expanduser("~/.rackspace_cloud_credentials")
 pyrax.set_credential_file(creds_file)
 clb = pyrax.cloud_loadbalancers
@@ -28,16 +31,17 @@ clb = pyrax.cloud_loadbalancers
 try:
     lb = clb.list()[0]
 except IndexError:
-    "You do not have any load balancers yet. Please create one and then re-run this script."
+    print("You do not have any load balancers yet.")
+    print("Please create one and then re-run this script.")
     sys.exit()
 
-print "Load Balancer:", lb
+print("Load Balancer:", lb)
 orig = lb.content_caching
-print "Current setting of content caching:", orig
-print
+print("Current setting of content caching:", orig)
+print()
 if orig:
-    print "Turning off..."
+    print("Turning off...")
 else:
-    print "Turning on..."
+    print("Turning on...")
 lb.content_caching = not orig
-print "New setting of content caching:", lb.content_caching
+print("New setting of content caching:", lb.content_caching)
