@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# Copyright 2012 Rackspace
+# Copyright (c)2012 Rackspace US, Inc.
 
 # All Rights Reserved.
 #
@@ -15,9 +15,13 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
+
+from __future__ import print_function
+
 import os
 import pyrax
 
+pyrax.set_setting("identity_type", "rackspace")
 creds_file = os.path.expanduser("~/.rackspace_cloud_credentials")
 pyrax.set_credential_file(creds_file)
 clb = pyrax.cloud_loadbalancers
@@ -30,7 +34,7 @@ clb = pyrax.cloud_loadbalancers
 #    id: None
 
 vip = clb.VirtualIP()
-print "Virtual IP (using defaults):", vip
-
+print("Virtual IP (using defaults):", vip
+)
 vip = clb.VirtualIP(type="SERVICENET", address="1.2.3.4", ipVersion="IPV4", id=999)
-print "Virtual IP (using supplied values):", vip
+print("Virtual IP (using supplied values):", vip)
