@@ -1,7 +1,9 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
+from __future__ import absolute_import, unicode_literals
 
 import random
+import time
 import unittest
 
 from mock import call
@@ -22,7 +24,7 @@ from pyrax.clouddns import RecordResultsIterator
 import pyrax.exceptions as exc
 import pyrax.utils as utils
 
-from tests.unit import fakes
+from pyrax import fakes
 
 example_uri = "http://example.com"
 
@@ -32,12 +34,15 @@ class CloudDNSTest(unittest.TestCase):
         super(CloudDNSTest, self).__init__(*args, **kwargs)
 
     def setUp(self):
+        super(CloudDNSTest, self).setUp()
         self.client = fakes.FakeDNSClient()
         self.client._manager = fakes.FakeDNSManager(self.client)
+        self.client._manager._set_delay(0.000001)
         self.domain = fakes.FakeDNSDomain()
         self.domain.manager = self.client._manager
 
     def tearDown(self):
+        super(CloudDNSTest, self).tearDown()
         self.client = None
         self.domain = None
 
@@ -47,13 +52,47 @@ class CloudDNSTest(unittest.TestCase):
             return domain
         clt = self.client
         dom = self.domain
-        clt._manager._get = Mock(return_value=dom)
         d1 = test(clt, dom)
-        d2 = test(clt, dom.id)
-        self.assertEqual(d1, d2)
+        self.assertEqual(d1, dom)
         self.assertTrue(isinstance(d1, CloudDNSDomain))
+
+    def test_assure_domain_id(self):
+        @assure_domain
+        def test(self, domain):
+            return domain
+        clt = self.client
+        dom = self.domain
+        clt._manager._get = Mock(return_value=dom)
+        d2 = test(clt, dom.id)
+        self.assertEqual(d2, dom)
         self.assertTrue(isinstance(d2, CloudDNSDomain))
 
+    def test_assure_domain_name(self):
+        @assure_domain
+        def test(self, domain):
+            return domain
+        clt = self.client
+        dom = self.domain
+        clt._manager._get = Mock(side_effect=exc.NotFound(""))
+        clt._manager._list = Mock(return_value=[dom])
+        d3 = test(clt, dom.name)
+        self.assertEqual(d3, dom)
+        self.assertTrue(isinstance(d3, CloudDNSDomain))
+
+    def test_set_timeout(self):
+        clt = self.client
+        mgr = clt._manager
+        new_timeout = random.randint(0, 99)
+        clt.set_timeout(new_timeout)
+        self.assertEqual(mgr._timeout, new_timeout)
+
+    def test_set_delay(self):
+        clt = self.client
+        mgr = clt._manager
+        new_delay = random.randint(0, 99)
+        clt.set_delay(new_delay)
+        self.assertEqual(mgr._delay, new_delay)
+
     def test_reset_paging_all(self):
         clt = self.client
         mgr = clt._manager
@@ -69,9 +108,9 @@ class CloudDNSTest(unittest.TestCase):
         mgr._paging["domain"]["total_entries"] = 99
         mgr._paging["domain"]["next_uri"] = "FAKE"
         exp_entries = random.randint(100, 200)
-        uri_string_next = utils.random_name()
+        uri_string_next = utils.random_unicode()
         next_uri = "%s/domains/%s" % (example_uri, uri_string_next)
-        uri_string_prev = utils.random_name()
+        uri_string_prev = utils.random_unicode()
         prev_uri = "%s/domains/%s" % (example_uri, uri_string_prev)
         body = {"totalEntries": exp_entries,
                 "links": [
@@ -81,8 +120,10 @@ class CloudDNSTest(unittest.TestCase):
                     "rel": "previous"}]}
         mgr._reset_paging("domain", body=body)
         self.assertEqual(mgr._paging["domain"]["total_entries"], exp_entries)
-        self.assertEqual(mgr._paging["domain"]["next_uri"], "/domains/%s" % uri_string_next)
-        self.assertEqual(mgr._paging["domain"]["prev_uri"], "/domains/%s" % uri_string_prev)
+        self.assertEqual(mgr._paging["domain"]["next_uri"], "/domains/%s" %
+                uri_string_next)
+        self.assertEqual(mgr._paging["domain"]["prev_uri"], "/domains/%s" %
+                uri_string_prev)
 
     def test_get_pagination_qs(self):
         clt = self.client
@@ -95,7 +136,7 @@ class CloudDNSTest(unittest.TestCase):
     def test_manager_list(self):
         clt = self.client
         mgr = clt._manager
-        fake_name = utils.random_name()
+        fake_name = utils.random_unicode()
         ret_body = {"domains": [{"name": fake_name}]}
         clt.method_get = Mock(return_value=({}, ret_body))
         ret = clt.list()
@@ -104,11 +145,12 @@ class CloudDNSTest(unittest.TestCase):
     def test_manager_list_all(self):
         clt = self.client
         mgr = clt._manager
-        fake_name = utils.random_name()
+        fake_name = utils.random_unicode()
         ret_body = {"domains": [{"name": fake_name}]}
-        uri_string_next = utils.random_name()
+        uri_string_next = utils.random_unicode()
         next_uri = "%s/domains/%s" % (example_uri, uri_string_next)
         mgr.count = 0
+
         def mock_get(uri):
             if mgr.count:
                 return ({}, ret_body)
@@ -205,12 +247,12 @@ class CloudDNSTest(unittest.TestCase):
     def test_manager_get(self):
         ret_body = {"recordsList": {
                 "records": [{
-                "accountId": "728829",
-                "created": "2012-09-21T21:32:27.000+0000",
-                "emailAddress": "me@example.com",
-                "id": "3448214",
-                "name": "example.com",
-                "updated": "2012-09-21T21:35:45.000+0000"
+                    "accountId": "728829",
+                    "created": "2012-09-21T21:32:27.000+0000",
+                    "emailAddress": "me@example.com",
+                    "id": "3448214",
+                    "name": "example.com",
+                    "updated": "2012-09-21T21:35:45.000+0000"
                 }]}}
         mgr = self.client._manager
         mgr.api.method_get = Mock(return_value=(None, ret_body))
@@ -225,12 +267,12 @@ class CloudDNSTest(unittest.TestCase):
         mgr.api.method_post = Mock(return_value=(None, ret_body))
         stat_body = {"status": "complete",
                 "response": {mgr.response_key: [{
-                "accountId": "728829",
-                "created": "2012-09-21T21:32:27.000+0000",
-                "emailAddress": "me@example.com",
-                "id": "3448214",
-                "name": "example.com",
-                "updated": "2012-09-21T21:35:45.000+0000"
+                    "accountId": "728829",
+                    "created": "2012-09-21T21:32:27.000+0000",
+                    "emailAddress": "me@example.com",
+                    "id": "3448214",
+                    "name": "example.com",
+                    "updated": "2012-09-21T21:35:45.000+0000"
                 }]}}
         mgr.api.method_get = Mock(return_value=(None, stat_body))
         dom = mgr._create("fake", {})
@@ -265,10 +307,16 @@ class CloudDNSTest(unittest.TestCase):
         BaseManager.findall.assert_called_once_with(foo="bar")
         BaseManager.findall = sav
 
-    def test_create_body(self):
+    def test_manager_empty_get_body_error(self):
         clt = self.client
-        fake_name = utils.random_name()
-        body = clt._create_body(fake_name, "fake@fake.com")
+        mgr = clt._manager
+        mgr.api.method_get = Mock(return_value=(None, None))
+        self.assertRaises(exc.ServiceResponseFailure, mgr.list)
+
+    def test_create_body(self):
+        mgr = self.client._manager
+        fake_name = utils.random_unicode()
+        body = mgr._create_body(fake_name, "fake@fake.com")
         self.assertEqual(body["domains"][0]["name"], fake_name)
 
     def test_async_call_body(self):
@@ -325,6 +373,17 @@ class CloudDNSTest(unittest.TestCase):
         clt.method_get.assert_called_once_with(massaged_uri)
         self.assertEqual(ret, ({}, get_resp))
 
+    def test_async_call_timeout(self):
+        clt = self.client
+        mgr = clt._manager
+        uri = "http://example.com"
+        callback_uri = "https://fake.example.com/status/fake"
+        clt.set_timeout(0.000001)
+        clt.method_get = Mock(return_value=({}, {"callbackUrl": callback_uri,
+                "status": "RUNNING"}))
+        self.assertRaises(exc.DNSCallTimedOut, mgr._async_call, uri,
+                method="GET")
+
     def test_async_call_error(self):
         clt = self.client
         mgr = clt._manager
@@ -378,25 +437,27 @@ class CloudDNSTest(unittest.TestCase):
     def test_export_domain(self):
         clt = self.client
         dom = self.domain
-        export = utils.random_name()
+        export = utils.random_unicode()
         clt._manager._async_call = Mock(return_value=({}, {"contents": export}))
         ret = clt.export_domain(dom)
         uri = "/domains/%s/export" % dom.id
-        clt._manager._async_call.assert_called_once_with(uri, error_class=exc.NotFound, method="GET")
+        clt._manager._async_call.assert_called_once_with(uri,
+                error_class=exc.NotFound, method="GET")
         self.assertEqual(ret, export)
 
     def test_import_domain(self):
         clt = self.client
         mgr = clt._manager
-        data = utils.random_name()
+        data = utils.random_unicode()
         mgr._async_call = Mock(return_value=({}, "fake"))
-        req_body = {"domains" : [{
-                "contentType" : "BIND_9",
-                "contents" : data,
+        req_body = {"domains": [{
+                "contentType": "BIND_9",
+                "contents": data,
                 }]}
         ret = clt.import_domain(data)
-        mgr._async_call.assert_called_once_with("/domains/import", method="POST",
-                body=req_body, error_class=exc.DomainCreationFailed)
+        mgr._async_call.assert_called_once_with("/domains/import",
+                method="POST", body=req_body,
+                error_class=exc.DomainCreationFailed)
 
     def test_update_domain_empty(self):
         self.assertRaises(exc.MissingDNSSettings, self.client.update_domain,
@@ -407,12 +468,12 @@ class CloudDNSTest(unittest.TestCase):
         dom = self.domain
         mgr = clt._manager
         emailAddress = None
-        comment = utils.random_name()
+        comment = utils.random_unicode()
         ttl = 666
         mgr._async_call = Mock(return_value=({}, "fake"))
         uri = "/domains/%s" % utils.get_id(dom)
-        req_body = {"comment" : comment,
-                "ttl" : ttl,
+        req_body = {"comment": comment,
+                "ttl": ttl,
                 }
         ret = clt.update_domain(dom, emailAddress, ttl, comment)
         mgr._async_call.assert_called_once_with(uri, method="PUT",
@@ -443,8 +504,8 @@ class CloudDNSTest(unittest.TestCase):
         clt = self.client
         mgr = clt._manager
         dom = self.domain
-        clt.method_get = Mock(return_value=({}, {}))
-#        uri = "/domains/%s/subdomains" % utils.get_id(dom)
+        resp_body = {'Something': 'here'}
+        clt.method_get = Mock(return_value=({}, resp_body))
         uri = "/domains?name=%s&limit=5" % dom.name
         clt.list_subdomains(dom, limit=5)
         clt.method_get.assert_called_once_with(uri)
@@ -453,7 +514,8 @@ class CloudDNSTest(unittest.TestCase):
         clt = self.client
         mgr = clt._manager
         dom = self.domain
-        clt.method_get = Mock(return_value=({}, {}))
+        resp_body = {'Something': 'here'}
+        clt.method_get = Mock(return_value=({}, resp_body))
         uri = "/domains/%s/records" % utils.get_id(dom)
         clt.list_records(dom)
         clt.method_get.assert_called_once_with(uri)
@@ -466,6 +528,7 @@ class CloudDNSTest(unittest.TestCase):
         uri = "/domains/%s/records?type=%s" % (utils.get_id(dom), typ)
         ret_body = {"records": [{"type": typ}]}
         mgr.count = 0
+
         def mock_get(uri):
             if mgr.count:
                 return ({}, ret_body)
@@ -487,11 +550,12 @@ class CloudDNSTest(unittest.TestCase):
         mgr = clt._manager
         dom = self.domain
         typ = "A"
-        nm = utils.random_name()
+        nm = utils.random_unicode()
         data = "0.0.0.0"
-        clt.method_get = Mock(return_value=({}, {}))
-        uri = "/domains/%s/records?type=%s&name=%s&data=%s" % (utils.get_id(dom),
-                typ, nm, data)
+        resp_body = {"Something": "here"}
+        clt.method_get = Mock(return_value=({}, resp_body))
+        uri = "/domains/%s/records?type=%s&name=%s&data=%s" % (
+                utils.get_id(dom), typ, nm, data)
         clt.search_records(dom, typ, name=nm, data=data)
         clt.method_get.assert_called_once_with(uri)
 
@@ -500,7 +564,7 @@ class CloudDNSTest(unittest.TestCase):
         mgr = clt._manager
         dom = self.domain
         typ = "A"
-        nm = utils.random_name()
+        nm = utils.random_unicode()
         data = "0.0.0.0"
         ret_body = {"records": [{
                 "accountId": "728829",
@@ -511,8 +575,8 @@ class CloudDNSTest(unittest.TestCase):
                 "updated": "2012-09-21T21:35:45.000+0000"
                 }]}
         clt.method_get = Mock(return_value=({}, ret_body))
-        uri = "/domains/%s/records?type=%s&name=%s&data=%s" % (utils.get_id(dom),
-                typ, nm, data)
+        uri = "/domains/%s/records?type=%s&name=%s&data=%s" % (
+                utils.get_id(dom), typ, nm, data)
         clt.find_record(dom, typ, name=nm, data=data)
         clt.method_get.assert_called_once_with(uri)
 
@@ -521,12 +585,12 @@ class CloudDNSTest(unittest.TestCase):
         mgr = clt._manager
         dom = self.domain
         typ = "A"
-        nm = utils.random_name()
+        nm = utils.random_unicode()
         data = "0.0.0.0"
         ret_body = {"records": []}
         clt.method_get = Mock(return_value=({}, ret_body))
-        uri = "/domains/%s/records?type=%s&name=%s&data=%s" % (utils.get_id(dom),
-                typ, nm, data)
+        uri = "/domains/%s/records?type=%s&name=%s&data=%s" % (
+                utils.get_id(dom), typ, nm, data)
         self.assertRaises(exc.DomainRecordNotFound, clt.find_record, dom, typ,
                 name=nm, data=data)
 
@@ -535,7 +599,7 @@ class CloudDNSTest(unittest.TestCase):
         mgr = clt._manager
         dom = self.domain
         typ = "A"
-        nm = utils.random_name()
+        nm = utils.random_unicode()
         data = "0.0.0.0"
         ret_body = {"records": [{
                 "accountId": "728829",
@@ -544,8 +608,7 @@ class CloudDNSTest(unittest.TestCase):
                 "id": "3448214",
                 "name": "example.com",
                 "updated": "2012-09-21T21:35:45.000+0000"
-                }, {
-                "accountId": "728829",
+                }, {"accountId": "728829",
                 "created": "2012-09-21T21:32:27.000+0000",
                 "emailAddress": "me@example.com",
                 "id": "3448214",
@@ -553,8 +616,8 @@ class CloudDNSTest(unittest.TestCase):
                 "updated": "2012-09-21T21:35:45.000+0000"
                 }]}
         clt.method_get = Mock(return_value=({}, ret_body))
-        uri = "/domains/%s/records?type=%s&name=%s&data=%s" % (utils.get_id(dom),
-                typ, nm, data)
+        uri = "/domains/%s/records?type=%s&name=%s&data=%s" % (
+                utils.get_id(dom), typ, nm, data)
         self.assertRaises(exc.DomainRecordNotUnique, clt.find_record, dom, typ,
                 name=nm, data=data)
 
@@ -567,31 +630,45 @@ class CloudDNSTest(unittest.TestCase):
         uri = "/domains/%s/records" % utils.get_id(dom)
         clt.add_records(dom, rec)
         mgr._async_call.assert_called_once_with(uri, method="POST",
-                body={"records": [rec]}, error_class=exc.DomainRecordAdditionFailed,
+                body={"records": [rec]},
+                error_class=exc.DomainRecordAdditionFailed,
                 has_response=False)
 
+    def test_get_record(self):
+        clt = self.client
+        mgr = clt._manager
+        dom = self.domain
+        nm = utils.random_unicode()
+        rec_id = utils.random_unicode()
+        rec_dict = {"id": rec_id, "name": nm}
+        mgr.api.method_get = Mock(return_value=(None, rec_dict))
+        ret = clt.get_record(dom, rec_id)
+        mgr.api.method_get.assert_called_once_with("/%s/%s/records/%s" %
+                (mgr.uri_base, dom.id, rec_id))
+
     def test_update_record(self):
         clt = self.client
         mgr = clt._manager
         dom = self.domain
-        nm = utils.random_name()
-        rec = fakes.FakeDNSRecord(mgr, {"id": utils.random_name(),
-                "name": nm})
+        nm = utils.random_unicode()
+        rec_id = utils.random_unicode()
+        rec = fakes.FakeDNSRecord(mgr, {"id": rec_id, "name": nm})
         ttl = 9999
         data = "0.0.0.0"
         mgr._async_call = Mock(return_value=({}, {}))
-        uri = "/domains/%s/records/%s" % (utils.get_id(dom), utils.get_id(rec))
-        req_body = {"name": nm, "data": data, "ttl": ttl}
+        uri = "/domains/%s/records" % utils.get_id(dom)
+        req_body = {"id": rec_id, "name": nm, "data": data, "ttl": ttl}
         clt.update_record(dom, rec, data=data, ttl=ttl)
         mgr._async_call.assert_called_once_with(uri, method="PUT",
-                body=req_body, error_class=exc.DomainRecordUpdateFailed,
+                body={"records": [req_body]},
+                error_class=exc.DomainRecordUpdateFailed,
                 has_response=False)
 
     def test_delete_record(self):
         clt = self.client
         mgr = clt._manager
         dom = self.domain
-        rec = CloudDNSRecord(mgr, {"id": utils.random_name()})
+        rec = CloudDNSRecord(mgr, {"id": utils.random_unicode()})
         mgr._async_call = Mock(return_value=({}, {}))
         uri = "/domains/%s/records/%s" % (utils.get_id(dom), utils.get_id(rec))
         clt.delete_record(dom, rec)
@@ -604,7 +681,7 @@ class CloudDNSTest(unittest.TestCase):
         mgr = clt._manager
         device = fakes.FakeDNSDevice()
         typ = mgr._resolve_device_type(device)
-        self.assertEqual(typ, "server")
+        self.assertEqual(typ, "loadbalancer")
         device = fakes.FakeLoadBalancer()
         typ = mgr._resolve_device_type(device)
         self.assertEqual(typ, "loadbalancer")
@@ -613,20 +690,8 @@ class CloudDNSTest(unittest.TestCase):
         clt = self.client
         mgr = clt._manager
         device = object()
-        self.assertRaises(exc.InvalidDeviceType, mgr._resolve_device_type, device)
-
-    def test_get_ptr_details_server(self):
-        clt = self.client
-        mgr = clt._manager
-        dvc = fakes.FakeDNSDevice()
-        dvc_type = "server"
-        sav = pyrax._get_service_endpoint
-        pyrax._get_service_endpoint = Mock(return_value=example_uri)
-        expected_href = "%s/servers/%s" % (example_uri, dvc.id)
-        href, svc_name = mgr._get_ptr_details(dvc, dvc_type)
-        self.assertEqual(svc_name, "cloudServersOpenStack")
-        self.assertEqual(href, expected_href)
-        pyrax._get_service_endpoint = sav
+        self.assertRaises(exc.InvalidDeviceType, mgr._resolve_device_type,
+                device)
 
     def test_get_ptr_details_lb(self):
         clt = self.client
@@ -680,8 +745,8 @@ class CloudDNSTest(unittest.TestCase):
         mgr._get_ptr_details = Mock(return_value=(href, svc_name))
         mgr._async_call = Mock(return_value=({}, {"records": []}))
         clt.add_ptr_records(dvc, rec)
-        mgr._async_call.assert_called_once_with(uri, body=body, error_class=exc.PTRRecordCreationFailed,
-                method="POST")
+        mgr._async_call.assert_called_once_with(uri, body=body,
+                error_class=exc.PTRRecordCreationFailed, method="POST")
 
     def test_update_ptr_record(self):
         clt = self.client
@@ -689,7 +754,7 @@ class CloudDNSTest(unittest.TestCase):
         dvc = fakes.FakeDNSDevice()
         href = "%s/%s" % (example_uri, dvc.id)
         svc_name = "cloudServersOpenStack"
-        ptr_record = fakes.FakeDNSPTRRecord({"id": utils.random_name()})
+        ptr_record = fakes.FakeDNSPTRRecord({"id": utils.random_unicode()})
         ttl = 9999
         data = "0.0.0.0"
         long_comment = "x" * 200
@@ -698,13 +763,15 @@ class CloudDNSTest(unittest.TestCase):
         rec = {"name": nm, "id": ptr_record.id, "type": "PTR", "data": data,
                 "ttl": ttl, "comment": trim_comment}
         uri = "/rdns"
-        body = {"recordsList": {"records": [rec]}, "link": {"content": "", "href": href, "rel": svc_name}}
+        body = {"recordsList": {"records": [rec]}, "link": {"content": "",
+                "href": href, "rel": svc_name}}
         mgr._get_ptr_details = Mock(return_value=(href, svc_name))
         mgr._async_call = Mock(return_value=({}, {"records": []}))
-        clt.update_ptr_record(dvc, ptr_record, domain_name=nm, data=data, ttl=ttl,
-                comment=long_comment)
-        mgr._async_call.assert_called_once_with(uri, body=body, error_class=exc.PTRRecordUpdateFailed,
-                method="PUT", has_response=False)
+        clt.update_ptr_record(dvc, ptr_record, domain_name=nm, data=data,
+                ttl=ttl, comment=long_comment)
+        mgr._async_call.assert_called_once_with(uri, body=body,
+                error_class=exc.PTRRecordUpdateFailed, method="PUT",
+                has_response=False)
 
     def test_delete_ptr_records(self):
         clt = self.client
@@ -717,12 +784,13 @@ class CloudDNSTest(unittest.TestCase):
         mgr._get_ptr_details = Mock(return_value=(href, svc_name))
         mgr._async_call = Mock(return_value=({}, {"records": []}))
         ret = clt.delete_ptr_records(dvc, ip_address=ip_address)
-        mgr._async_call.assert_called_once_with(uri, error_class=exc.PTRRecordDeletionFailed,
+        mgr._async_call.assert_called_once_with(uri,
+                error_class=exc.PTRRecordDeletionFailed,
                 method="DELETE", has_response=False)
 
     def test_get_absolute_limits(self):
         clt = self.client
-        rand_limit = utils.random_name()
+        rand_limit = utils.random_unicode()
         resp = {"limits": {"absolute": rand_limit}}
         clt.method_get = Mock(return_value=({}, resp))
         ret = clt.get_absolute_limits()
@@ -761,7 +829,7 @@ class CloudDNSTest(unittest.TestCase):
     def test_iter_items_first_fetch(self):
         clt = self.client
         mgr = clt._manager
-        fake_name = utils.random_name()
+        fake_name = utils.random_unicode()
         ret_body = {"domains": [{"name": fake_name}]}
         clt.method_get = Mock(return_value=({}, ret_body))
         res_iter = DomainResultsIterator(mgr)
@@ -772,7 +840,7 @@ class CloudDNSTest(unittest.TestCase):
     def test_iter_items_next_fetch(self):
         clt = self.client
         mgr = clt._manager
-        fake_name = utils.random_name()
+        fake_name = utils.random_unicode()
         ret_body = {"domains": [{"name": fake_name}]}
         clt.method_get = Mock(return_value=({}, ret_body))
         res_iter = DomainResultsIterator(mgr)
@@ -799,8 +867,14 @@ class CloudDNSTest(unittest.TestCase):
         res_iter = RecordResultsIterator(mgr)
         self.assertEqual(res_iter.paging_service, "record")
 
-
-
+    # patch BaseClients method_get to make it always return an empty
+    # body. client method_get uses super to get at BaseClient's
+    # method_get.
+    @patch.object(pyrax.client.BaseClient, "method_get",
+            new=lambda x, y: (None, None))
+    def test_client_empty_get_body_error(self):
+        clt = self.client
+        self.assertRaises(exc.ServiceResponseFailure, clt.get_absolute_limits)
 
 
 if __name__ == "__main__":
